\name{extract_bic}
\alias{extract_bic}
\title{Extraction of Biclusters}
\description{

  \code{extract_bic}: \R implementation of  \code{extract_bic}.
}
\usage{

extract_bic(L,Z,thresZ=0.5,thresL=NULL,lapla=NULL,Psi=NULL)

}
\arguments{
  \item{L}{loading, left matrix.}
  \item{Z}{factor, right matrix.}
  \item{thresZ}{threshold for sample belonging to bicluster (default 0.5).}
  \item{thresL}{threshold for loading belonging to bicluster (if not
    given it is estimated).}
  \item{lapla}{inverse variance of the variational approximation for
    each sample and each factor.}
  \item{Psi}{noise variance vector for observations where independent noise is
    asumed.}
}
\details{

  Essentially the model is the sum of
  outer products of vectors. The number of summands  \eqn{p}
  is the number of biclusters.


  \deqn{X \ = \ L \ Z \ + \ U}

  \deqn{X \ = \ \sum_{i=1}^{p} L_i \
                   \left(Z_i \right)^T \ + \ U}

   The hidden dimension \eqn{p} is used for kmeans clustering of
   \eqn{L_i} and \eqn{Z_i }.

   \eqn{U } is the Gaussian noise with a diagonal covariance matrix
   which entries are given by \code{Psi}.
  
   The \eqn{Z } is locally approximated by a Gaussian with inverse
   variance given by  \code{lapla}.
   
   Using these values we can computer
   for each  \eqn{j} the variance \eqn{ Z_i} given \eqn{ x_j}. Here

   \deqn{ x_j \ = \ L \ z_j \ + \ u_j}

   This variance can be used to determine the information content
   of a bicluster.
   
   
   The \eqn{L_i } and \eqn{Z_i } are used to
   extract the bicluster \eqn{i}, where a threshold determines which
   observations and which samples belong the the bicluster.



   In \code{bic} the biclusters are extracted according to the
   largest absolute values of the component \eqn{i}, i.e.
   the largest values of \eqn{L_i } and the
   largest values of \eqn{Z_i }. The factors \eqn{ Z_i}
   are normalized to variance 1.

   
   The components of \code{bic} are
   \code{binp}, \code{bixv},
   \code{bixn}, \code{biypv}, and \code{biypn}.
   

   \code{binp} give the size of the bicluster: number observations and
   number samples.
   \code{bixv} gives the values of the extracted
   observations that have absolute
   values above a threshold. They are sorted. 
   \code{bixn} gives  the extracted observation names (e.g. gene names).
   \code{biypv} gives the values of the extracted samples that have
   absolute values above a threshold.  They are sorted.
   \code{biypn} gives  the names of the extracted samples (e.g. sample names).

   In \code{bicopp} the opposite cluster to the biclusters are
   give. Opposite means that the negative pattern is present.
   
   The components of opposite clusters \code{bicopp} are
   \code{binn}, \code{bixv},
   \code{bixn}, \code{biypnv}, and \code{biynn}.
   
    \code{binp} give the size of the opposite bicluster: number observations and
   number samples.
   \code{bixv} gives the values of the extracted
   observations that have absolute
   values above a threshold. They are sorted. 
   \code{bixn} gives  the extracted observation names (e.g. gene names).
   \code{biynv} gives the values of the opposite extracted samples that have
   absolute values above a threshold.  They are sorted.
   \code{biynn} gives  the names of the opposite
   extracted samples (e.g. sample names).
   
   That means the samples are divided into two groups where one group
   shows large positive values and the other group has negative values
   with large absolute values. That means a observation pattern can be
   switched on or switched off relative to the average value.

   \code{numn} gives the indexes of \code{bic} with components:
   \code{numng} = \code{bix} and \code{numnp}  =  \code{biypn}.
   
   \code{numn} gives the indexes of \code{bicopp} with components:
   \code{numng} = \code{bix} and \code{numnn}  =  \code{biynn}.
   


  Implementation in \R.
}

\value{
  \item{bic}{extracted biclusters.}
  \item{numn}{indexes for the extracted biclusters.}
  \item{bicopp}{extracted opposite biclusters.}
  \item{numnopp}{indexes for the extracted opposite biclusters.}
  \item{avini}{average over  \eqn{j} of the
    variance \eqn{ Z_i} given  \eqn{ x_j}.}
  \item{ini}{for each  \eqn{j} the variance \eqn{ Z_i} given  \eqn{ x_j}.}
    }
\seealso{
\code{\link{fabi}},
\code{\link{fabia}},
\code{\link{fabiap}},
\code{\link{fabias}},
\code{\link{fabiasp}},
\code{\link{mfsc}},
\code{\link{nmfdiv}},
\code{\link{nmfeu}},
\code{\link{nmfsc}},
\code{\link{nprojfunc}},
\code{\link{projfunc}},
\code{\link{make_fabi_data}},
\code{\link{make_fabi_data_blocks}},
\code{\link{make_fabi_data_pos}},
\code{\link{make_fabi_data_blocks_pos}},
\code{\link{extract_plot}},
\code{\link{myImagePlot}},
\code{\link{PlotBicluster}},
\code{\link{Breast_A}},
\code{\link{DLBCL_B}},
\code{\link{Multi_A}},
\code{\link{fabiaDemo}},
\code{\link{fabiaVersion}}
}
\author{Sepp Hochreiter}
\examples{

#---------------
# TEST
#---------------

dat <- make_fabi_data_blocks(n = 100,l= 50,p = 3,f1 = 5,f2 = 5,
  of1 = 5,of2 = 10,sd_noise = 3.0,sd_z_noise = 0.2,mean_z = 2.0,
  sd_z = 1.0,sd_l_noise = 0.2,mean_l = 3.0,sd_l = 1.0)

X <- dat[[1]]
Y <- dat[[2]]


resEx <- fabia(X,20,0.1,1.0,1.0,3)

rEx <- extract_bic(resEx$L,resEx$Z,lapla=resEx$lapla,Psi=resEx$Psi)

rEx$bic[1,]
rEx$bic[2,]
rEx$bic[3,]


\dontrun{
#---------------
# DEMO1
#---------------

dat <- make_fabi_data_blocks(n = 1000,l= 100,p = 10,f1 = 5,f2 = 5,
  of1 = 5,of2 = 10,sd_noise = 3.0,sd_z_noise = 0.2,mean_z = 2.0,
  sd_z = 1.0,sd_l_noise = 0.2,mean_l = 3.0,sd_l = 1.0)

X <- dat[[1]]
Y <- dat[[2]]


resToy <- fabia(X,200,0.4,1.0,1.0,13)

rToy <- extract_bic(resToy$L,resToy$Z,lapla=resToy$lapla,Psi=resToy$Psi)

rToy$avini

rToy$bic[1,]
rToy$bic[2,]
rToy$bic[3,]

#---------------
# DEMO2
#---------------

data(Breast_A)

X <- as.matrix(XBreast)

resBreast <- fabia(X,200,0.1,1.0,1.0,5)

rBreast <- extract_bic(resBreast$L,resBreast$Z,lapla=resBreast$lapla,Psi=resBreast$Psi)

rBreast$avini

rBreast$bic[1,]
rBreast$bic[2,]
rBreast$bic[3,]

}
}
\concept{biclustering}
\concept{sparse coding}
\concept{sparse matrix factorization}
\concept{non-negative matrix factorization}
